function [intFF, intDFDF] = calc_intFF_intDFDF(shape, fem_deg)
% function [intFF, intDFDF] = calc_intFF_intDFDF(shape, fem_deg)
% calc the integral of F(i) * F(j) and DF(i) * DF(j)

%      1, 2, 3, 4,   5,   6
% u = [0, 0, 1, 0,   0.5, 0.5];
% v = [0, 1, 0, 0.5, 0.5, 0];
%
% 2
% |\
% 4 5
% |  \
% 1-6-3
%
%    2
%    /\
%   4--5
%  / \/ \
% 1--6---3

% calc unit integrals
[unit_intFF, unit_intFuFu, unit_intFuFv, unit_intFvFu, unit_intFvFv] = int_form_func(fem_deg);

[AB_xyz, AC_xyz] = tri_edge_xyz(shape.TRIV, shape.X, shape.Y, shape.Z);
ABxAC = norm_row(cross(AB_xyz, AC_xyz, 2));
ABoAB = dot(AB_xyz, AB_xyz, 2) ./ ABxAC;
ACoAC = dot(AC_xyz, AC_xyz, 2) ./ ABxAC;
ABoAC = dot(AB_xyz, AC_xyz, 2) ./ ABxAC;

intFF = ABxAC * unit_intFF(:)';

intDFDF = ABoAB * unit_intFuFu(:)' + ACoAC * unit_intFvFv(:)' - ABoAC * (unit_intFuFv(:)' + unit_intFvFu(:)');
end %function [intFF, intDFDF] = calc_intFF_intDFDF(shape, fem_deg)
