function [g] = calc_sym_g()
% function [g] = calc_sym_g()
% g is 2x2
%
%
%   |               |
% v |     T(u,v)  y |      C
% 1 |\    ----->    |      /\
%   | \             |   v /  \
%   |__\__          |  A /-->-\ B
%      1 u          |      u
%                   |--------------------
%                                     x 
% A, B and C are the vertices of the original trianlge (v1, v2, v3) on a transformed axes
% the origin is at vertex v1;
% the x axis is v2 - v1;
% the z axis is (v2 - v1) x (v3 - v1)
% the y axis is z x x
% 
% so A(x,y) = 0 and the z component of A, B, and C is defined by the parab fit
%
% in the x and y dimensions: T(u,v) = u*B + v*C
% so
% x(u,v) = u*B1 + v*C1
% y(u,v) = u*B2 + v*C2
% z(x,y) = a + b*x + c*y + d*x^2 + e*x*y + f*y^2 as calculated in tri_parab_fit

syms a b c d e f real; % parab coeff
syms B1 B2 C1 C2 real; % points B=(B1, B2) and C=(C1, C2); 
syms u v real;

% calc g
x = u * B1 + v * C1;
y = u * B2 + v * C2;
z = a + b*x + c*y + d*x^2 + e*x*y + f*y^2;
T = [x, y, z];
Tu = diff(T, 'u');
Tv = diff(T, 'v');
TuoTu = dot(Tu, Tu);
TuoTv = dot(Tu, Tv);
TvoTv = dot(Tv, Tv);
g = [TuoTu, TuoTv;...
    TuoTv, TvoTv];

% z(x,y) = a + b*x + c*y + d*x^2 + e*x*y + f*y^2 as calculated in tri_parab_fit
% z(u,v) = a + b*(u*B1 + v*C1) + c*(u*B2 + v*C2) + d*(u*B1 + v*C1)^2
%           + e*(u*B1 + v*C1)*(u*B2 + v*C2) + f*(u*B2 + v*C2)^2
% z(u,v) = a + u*(b*B1+c*B2) + v*(b*C1+c*C2) + 
%           + u^2*(d*B1^2+e*B1*B2 + f*B2^2) + u*v*(2d*B1*C1+e*(B1*C2 + B2*C1)+2f*B2*C2) 
%           + v^2*(d*C1^2 + e*C1*C2+ f*C2^2)
%
% dT/du = (B1, B2, (b*B1+c*B2) + u*2*(d*B1^2+e*B1*B2+f*B2^2) + v*(2*d*B1*C1+e*(B1*C2+B2*C1)+2*f*B2*C2))
% dT/dv = (C1, C2, (b*C1+c*C2) + v*2*(d*C1^2+e*C1*C2+f*C2^2) + u*(2*d*B1*C1+e*(B1*C2+B2*C1)+2*f*B2*C2))

% dT/du = (B1, B2, D1 + u*D2 + v*D3)
% dT/dv = (C1, C2, D4 + u*D5 + v*D6)

% D1 = b*B1+c*B2;
% D4 = b*C1+c*C2; 
% D2 = 2*(d*B1^2 + e*B1*B2 + f*B2^2);
% D6 = 2*(d*C1^2 + e*C1*C2 + f*C2^2);
% D3 = 2*d*B1*C1 + e*(B1*C2 + B2*C1) + 2*f*B2*C2;
% D5 = 2*d*B1*C1 + e*(B1*C2 + B2*C1) + 2*f*B2*C2;

% <dT/du, dT/du> = B1^2 + B2^2 + D1^2 + u*(2*D1*D2) + v*(2*D1*D3) + u^2*(D2^2)+u*v*(2*D2*D3)+v^2*(D3^2);
% <dT/dv, dT/dv> = C1^2 + C2^2 + D4^2 + u*(2*D4*D5) + v*(2*D4*D6) + u^2*(D5^2)+u*v*(2*D5*D6)+v^2*(D6^2);
% <dT/du, dT/dv> = B1*C1 + B2*C2 + D1*D4 + u*(D1*D5+D2*D4) + v*(D1*D6+D3*D4) + u^2*(D2*D5)+u*v*(D3*D5+D2*D6)+v^2*(D3*D6);

% <dT/du, dT/du> = E1 + u*E2 + v*E3 + u^2*E4 + u*v*E5 + v^2*E6;
% <dT/dv, dT/dv> = E7 + u*E8 + v*E9 + u^2*E10 +u*v*E11 + v^2*E12;
% <dT/du, dT/dv> = E13 + u*E14 + v*E15 + u^2*E16+u*v*E17+v^2*E18;

% E1 = B1^2 + B2^2 + D1^2;
% E2 = 2*D1*D2;
% E3 = 2*D1*D3;
% E4 = D2^2;
% E5 = 2*D2*D3;
% E6 = D3^2;
% 
% E7 = C1^2 + C2^2 + D4^2;
% E8 = 2*D4*D5;
% E9 = 2*D4*D6;
% E10 = D5^2;
% E11 = 2*D5*D6;
% E12 = D6^2;
% 
% E13 = B1*C1 + B2*C2 + D1*D4;
% E14 = D1*D5+D2*D4;
% E15 = D1*D6+D3*D4;
% E16 = D2*D5;
% E17 = D3*D5+D2*D6;
% E18 = D3*D6;
% 
% 
