function [FF, FuFu, FuFv, FvFu, FvFv] = form_func(fem_deg)
% [FF, FuFu, FuFv, FvFu, FvFv] = form_func(fem_deg)
% form functions over unit trianlge
% FF(i, j) = F(i) * F(j)
% F(i) has the value of 1 at point i and zero on all other points
% fem_deg is the fem_degree of F
% the u and v coordinates of the points are:
%      1  2  3  4    5    6
% u = [0, 0, 1, 0,   0.5, 0.5];
% v = [0, 1, 0, 0.5, 0.5, 0];
syms u v real;

switch(fem_deg)
    case(1)
        u_ = [0, 0, 1];
        v_ = [0, 1, 0];
        P_ = [ones(size(u_)); u_; v_];
        p = [1, u, v]';
    case(2)
        u_ = [0, 0, 1, 0,   0.5, 0.5];
        v_ = [0, 1, 0, 0.5, 0.5, 0];
        P_ = [ones(size(u_)); u_; v_; u_.^2; u_.*v_; v_.^2];
        p = [1, u, v, u.^2, u.*v, v.^2]';
    otherwise
        assert(0);
end

C = inv(P_); % C * P_ = I

F = C * p;

Fu = diff(F, 'u');
Fv = diff(F, 'v');

FF = F * F';
FuFu = Fu * Fu';
FuFv = Fu * Fv';
FvFu = Fv * Fu';
FvFv = Fv * Fv';

end % [FF, FuFu, FuFv, FvFu, FvFv] = form_func(fem_deg)

