%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [shape] = calc_mesh(shape)

switch(shape.type)
    case('alex')
        load 0001.null.0.mat;
        TRIV = shape.TRIV;
        X = shape.X;
        Y = shape.Y;
        Z = shape.Z;
    case('rec_with_tail')
        %         ----------
        %         |        |
        %         |        |
        %         |        |
        %         -------| |
        %                | |
        %                | |
        %                ---

        % check input
        assert(shape.y0_rec < shape.y1_rec);
        assert(shape.y1_rec + shape.dy == shape.y0_tail);
        assert(shape.y0_tail < shape.y1_tail);

        assert(shape.x0_rec <= shape.x0_tail);
        assert(shape.x0_tail < shape.x1_tail);
        assert(shape.x1_tail == shape.x1_rec);

        [X_rec, Y_rec, Z_rec, Tri_rec] = calc_mesh_rec(shape.x0_rec, shape.dx, shape.x1_rec, shape.y0_rec, shape.dy, shape.y1_rec);
        [X_tail, Y_tail, Z_tail, Tri_tail] = calc_mesh_rec(shape.x0_tail, shape.dx, shape.x1_tail, shape.y0_tail, shape.dy, shape.y1_tail);

        %glue rec to tail

        [num_rows_rec, num_cols_rec] = size(X_rec);
        [num_rows_tail, num_cols_tail] = size(X_tail);

        last_row_rec_ind = sub2ind(size(X_rec), repmat(num_rows_rec, num_cols_tail, 1), [(num_cols_rec - num_cols_tail + 1):num_cols_rec]');
        first_row_tail_ind = sub2ind(size(X_tail), repmat(1, num_cols_tail, 1), [1:num_cols_tail]') + numel(X_rec);

        Tri_glue1 = [last_row_rec_ind(1:(end - 1)), first_row_tail_ind(2:end), last_row_rec_ind(2:end)];
        Tri_glue2 = [last_row_rec_ind(1:(end - 1)), first_row_tail_ind(2:end), first_row_tail_ind(1:(end - 1))];

        Tri_tail = Tri_tail + numel(X_rec);

        X = [X_rec(:); X_tail(:)];
        Y = [Y_rec(:); Y_tail(:)];
        Z = [Z_rec(:); Z_tail(:)];
        TRIV = [Tri_rec; Tri_tail; Tri_glue1; Tri_glue2];

        shape.ar = (shape.x1_rec - shape.x0_rec)*(shape.y1_rec - shape.y0_rec) + ...
            (shape.x1_tail - shape.x0_tail)*(shape.y1_tail - shape.y1_rec);

    case('dbg')
        % define
        % *-------*
        % | \   / |
        % |  \ /  |
        % |   *   |
        % |  / \  |
        % | /   \ |
        % |*------*
        XYZ = [
            0,  0, 0;
            1,  0, 0;
            -1,  0, 0;
            0,  1, 0;
            0, -1, 0];
        X = XYZ(:, 1);
        Y = XYZ(:, 2);
        Z = XYZ(:, 3);
        TRIV = [
            1, 2, 4;
            1, 2, 5;
            1, 5, 3;
            1, 3, 4];
    case('sphere')
        FV = sphere_tri('ico', shape.n_recurse, shape.r);
        X = FV.vertices(:, 1);
        Y = FV.vertices(:, 2);
        Z = FV.vertices(:, 3);
        TRIV = FV.faces;
        shape.ar = 4 * pi * shape.r^2;

    case('sphere_old')        % define a sphere

        [X,Y,Z] = sphere(shape.n); figure; surf(X,Y,Z);

        % delete the last redundant column
        X(:, end) = [];  Y(:, end) = [];  Z(:, end) = [];

        % delete north and south poles
        X([1, end], :) = [];  Y([1, end], :) = [];  Z([1, end], :) = [];

        % define a trianlger mesh

        %               Ind_4=(I-1,J+1)
        %              /|
        %             / |
        %            /__|
        % Ind_1=(I,J)  Ind_2=(I,J+1)
        %
        % Ind_1=(I,J)____ Ind_2=(I,J+1)
        %            |  /
        %            | /
        %            |/
        %            Ind_3=(I+1,J)

        [num_rows, num_cols] = size(X);
        siz = [num_rows, num_cols];

        [J, I] = meshgrid(1:num_cols, 1:num_rows);

        I = I(:);
        J = J(:);
        Jplus1 = J + 1; Jplus1(J == num_cols) = 1;

        Ind_1 = sub2ind(siz, I, J);
        Ind_2 = sub2ind(siz, I, Jplus1);
        Ind_3 = sub2ind(siz, min(I + 1, num_rows), J);

        %Ind_1 = sub2ind(siz, I, J);
        %Ind_2 = sub2ind(siz, I, Jplus1);
        Ind_4 = sub2ind(siz, max(I - 1, 1), Jplus1);

        % add north pole and south pole
        X = [X(:); 0; 0]; Y = [Y(:); 0; 0]; Z = [Z(:); -1; 1];

        % map north pole indexes to north_pole_idx
        north_pole_idx = numel(X) - 1;
        Ind_4(I == 1) = north_pole_idx;
        % map south pole indexes to south_pole_idx
        south_pole_idx = numel(X);
        Ind_3(I == num_rows) = south_pole_idx;

        Ind_A = [Ind_1; Ind_1];
        Ind_B = [Ind_2; Ind_2];
        Ind_C = [Ind_3; Ind_4];

        TRIV = [Ind_A, Ind_B, Ind_C];
    case('rec')
        [X, Y, Z, TRIV] = calc_mesh_rec(shape.x0, shape.dx, shape.x1, shape.y0, shape.dy, shape.y1);
        shape.ar = (shape.x1 - shape.x0)*(shape.y1 - shape.y0);

end

%figure; trimesh(TRIV, X, Y, Z);
%figure; patch('vertices',[X, Y, Z], 'faces', TRIV, 'facecolor',[1 1 1],'edgecolor',[0, 0, 0]);
shape.TRIV = TRIV; shape.X = X; shape.Y = Y; shape.Z = Z;

end % function [shape] = calc_mesh(shape)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [X, Y, Z, TRIV] = calc_mesh_rec(x0, dx, x1, y0, dy, y1)

[X, Y] = meshgrid(x0:dx:x1, y0:dy:y1);
[num_rows, num_cols] = size(X);
siz = [num_rows, num_cols];
Z = zeros(siz);

% \----
% |\  |
% | \ |
% |  \|
% ----\

[J, I] = meshgrid(1:(num_cols - 1), 1:(num_rows - 1));

I = I(:);
J = J(:);

% \----
%  \  |
%   \ |
%    \|

Ind_1 = sub2ind(size(X), I, J);
Ind_2 = sub2ind(size(X), I + 1, J + 1);
Ind_3 = sub2ind(size(X), I, J + 1);

% |\
% | \
% |  \
% ----\

%Ind_1 = sub2ind(siz, I, J);
%Ind_2 = sub2ind(siz, I + 1, J + 1);
Ind_4 = sub2ind(siz, I + 1, J);

TRIV = [[Ind_1; Ind_1], [Ind_2; Ind_2], [Ind_3; Ind_4]];
X = X(:); Y=Y(:); Z=Z(:);
end %function [X, Y, Z, TRIV] = calc_mesh_rec(x0, dx, x1, y0, dy, y1)

