function [g11, g12, g22, W] = tri_g(parab_coeff, B, C)
% function [g11, g12, g22] = tri_g(parab_coeff, B, C)
%
% parab_coeff is num_tri x 6;
% parab_coeff(t, :) = [a, b, c, d, e, f];
% z = a + b*x + c*y + d*x^2 + e*xy + f*y^2
% the origin of the parab fit is at vertex 1;
% the x axis is v2 - v1;
% the z axis is (v2 - v1) x (v3 - v1)
% the y axis is z x x
%
% % g11, g12 and g22 are num_tri x 3

% the formula came from the function calc_sym_g
syms u v real

[B1, B2] = deal(B(:,1), B(:, 2));
[C1, C2] = deal(C(:,1), C(:, 2));
clear B C;
[a,b,c,d,e,f] = ...
    deal(parab_coeff(:,1), parab_coeff(:,2), parab_coeff(:,3), parab_coeff(:,4), parab_coeff(:,5), parab_coeff(:,6));
clear parab_coeff;

display('in tri_g');

s = ... %[1, u, v]
[B1.*b + B2.*c, ...
    2*B1.*B1.*d + B1.*B2.*e + B1.*B2.*e + 2.*B2.*B2.*f, ...
    2*B1.*C1.*d + B1.*C2.*e + B2.*C1.*e + 2.*C2.*B2.*f];

t = ... %[1, u, v]
[C1.*b + C2.*c, ...
    2*B1.*C1.*d + B2.*C1.*e + B1.*C2.*e + 2*B2.*C2.*f, ...
    2*C1.*C1.*d + C1.*C2.*e + C1.*C2.*e + 2*C2.*C2.*f];

% ss = s.^2 [1, u, v, u^2, uv, v^2]
ss = [s(:,1).^2, 2*s(:,1).*s(:,2), 2*s(:,1).*s(:,3), s(:,2).^2, 2*s(:,2).*s(:,3), s(:,3).^2]; 

% tt = t.^2 [1, u, v, u^2, uv, v^2]
tt = [t(:,1).^2, 2*t(:,1).*t(:,2), 2*t(:,1).*t(:,3), t(:,2).^2, 2*t(:,2).*t(:,3), t(:,3).^2]; 

% st = s*t [1, u, v, u^2, uv, v^2]
st = [s(:,1).*t(:,1), ...
    s(:,1).*t(:,2)+s(:,2).*t(:,1), ...
    s(:,1).*t(:,3)+s(:,3).*t(:,1), ...
    s(:,2).*t(:,2), ...
    s(:,2).*t(:,3) + s(:,3).*t(:,2), ...
    s(:,3).*t(:,3)]; 

%g11 = B1.^2 + B2.^2 + s.^2;
%g11 [1, u, v, u^2, uv, v^2]
g11 = ss;
g11(:, 1) = g11(:, 1) + B1.^2 + B2.^2;

%g22 = C1.^2 + C2.^2 + t.^2;
%g22 [1, u, v, u^2, uv, v^2]
g22 = tt;
g22(:, 1) = g22(:, 1) + C1.^2 + C2.^2;

%g12 = B1.*C1 + B2.*C2 + s.* t;
%g12 [1, u, v, u^2, uv, v^2]
g12 = st;
g12(:, 1) = g12(:, 1) + B1.*C1 + B2.*C2;

% [1, u, v, u^2, uv, v^2, u^3, u^2v, uv^2, v^3, u^4, u^3v, u^2v^2, uv^3, v^4]
g11xg22 = [g11(:,1).*g22(:,1),... % 1
    g11(:,1).*g22(:,2) + g11(:,2).*g22(:,1), ... % u
    g11(:,1).*g22(:,3) + g11(:,3).*g22(:,1),... % v
    g11(:,1).*g22(:,4) + g11(:,4).*g22(:,1) + g11(:,2).*g22(:,2), ... % u^2
    g11(:,1).*g22(:,5) + g11(:,5).*g22(:,1) + g11(:,2).*g22(:,3)+g11(:,3).*g22(:,2), ... %uv
    g11(:,1).*g22(:,6) + g11(:,6).*g22(:,1) + g11(:,3).*g22(:,3),... %v^2
    g11(:,2).*g22(:,4)+g11(:,4).*g22(:,2), ... % u^3    
    g11(:,3).*g22(:,4)+g11(:,4).*g22(:,3)+g11(:,2).*g22(:,5)+g11(:,5).*g22(:,2), ... % u^2v        
    g11(:,2).*g22(:,6)+g11(:,6).*g22(:,2)+g11(:,3).*g22(:,5)+g11(:,5).*g22(:,3), ... % uv^2
    g11(:,3).*g22(:,6)+g11(:,6).*g22(:,3), ... % v^3    
    g11(:,4).*g22(:,4), ... % u^4        
    g11(:,4).*g22(:,5) + g11(:,5).*g22(:,4), ... % u^3v            
    g11(:,4).*g22(:,6) + g11(:,6).*g22(:,4) + g11(:,5).*g22(:,5), ... % u^2v^2
    g11(:,5).*g22(:,6) + g11(:,6).*g22(:,5), ... % uv^3    
    g11(:,6).*g22(:,6), ... %  v^4            
    ];

% [1, u, v, u^2, uv, v^2, u^3, u^2v, uv^2, v^3, u^4, u^3v, u^2v^2, uv^3, v^4]
g12xg12 = [g12(:,1).^2,... % 1
    2*g12(:,1).*g12(:,2), ... % u
    2*g12(:,1).*g12(:,3),... % v
    2*g12(:,1).*g12(:,4)+ g12(:,2).^2, ... % u^2
    2*g12(:,1).*g12(:,5) + 2*g12(:,2).*g12(:,3), ... %uv
    2*g12(:,1).*g12(:,6) + g12(:,3).^2,... %v^2
    2*g12(:,2).*g12(:,4), ... % u^3    
    2*g12(:,3).*g12(:,4) + 2*g12(:,2).*g12(:,5), ... % u^2v        
    2*g12(:,2).*g12(:,6) + 2*g12(:,3).*g12(:,5), ... % uv^2
    2*g12(:,3).*g12(:,6), ... % v^3    
    g12(:,4).^2, ... % u^4        
    2*g12(:,4).*g12(:,5), ... % u^3v            
    2*g12(:,4).*g12(:,6) + g12(:,5).^2, ... % u^2v^2
    2*g12(:,5).*g12(:,6), ... % uv^3    
    g12(:,6).^2, ... %  v^4            
    ];

% [1, u, v, u^2, uv, v^2, u^3, u^2v, uv^2, v^3, u^4, u^3v, u^2v^2, uv^3, v^4]
W = sqrt(g11xg22 - g12xg12); fixme

end % function [g11, g12, g22, W] = tri_g(parab_coeff, B, C)
