function [parab_coeff, B, C] = tri_parab_fit(TRIV, X, Y, Z)
% function [parab_coeff] = tri_parab_fit(TRIV, X, Y, Z)
% parab_coeff is num_tri x 6;
% parab_coeff(t, :) = [a, b, c, d, e, f];
% z = a + b*x + c*y + d*x^2 + e*xy + f*y^2
% the origin of the parab fit is at vertex 1;
% the x axis is v2 - v1;
% the z axis is (v2 - v1) x (v3 - v1)
% the y axis is z x x
%
% B, C are 3xnum_tri and they are the xyz coordinates of the transformed vertices

%   normalize AB and AC and calc the new axes of each trianlge: {AB, (ABxAC)xAB,ABxAC}
[XYZ_1, XYZ_2, XYZ_3] = tri_vert_xyz(TRIV, X, Y, Z);
XYZ_12 = XYZ_2 - XYZ_1;
XYZ_13 = XYZ_3 - XYZ_1;

axis1 = unit_vecs(XYZ_12, 2)';
axis3 = unit_vecs(cross(XYZ_12, XYZ_13), 2)';
axis2 = cross(axis3, axis1);

XYZ = [X, Y, Z];

num_tri = size(TRIV, 1);
parab_coeff = zeros(num_tri, 6);
B = zeros(num_tri, 3);
C = zeros(num_tri, 3);

for t=1:num_tri
    %   find the neighbors of TRIV(t, :)
    tri_idx = any(ismember(TRIV, TRIV(t, :)), 2);
    vert_idx = unique(TRIV(tri_idx, :));

    % set A to be the origin
    Origin = repmat(XYZ_1(t, :), size(vert_idx, 1), 1);
    xyz = (XYZ(vert_idx, :) - Origin)';

%     figure(1);
%     trimesh(TRIV(tri_idx, :), X, Y, Z);
%     grid on; xlabel('x'); ylabel('y'); zlabel('z');
%     hold off;

    % transform the axes to: {AB, (ABxAC)xAB, ABxAC}
    M = inv([axis1(:, t), axis2(:, t), axis3(:, t)]);
    xyz = M * xyz;
    
    B(t, :) = (M*XYZ_12(t, :)')';
    C(t, :) = (M*XYZ_13(t, :)')';

    % calc the best fit z = a + bx + cy + dx^2 + exy + fy^2
    x = xyz(1, :)';
    y = xyz(2, :)';
    z = xyz(3, :)';
    x2 = x.^2;
    xy = x.*y;
    y2 = y.^2;
    ons = ones(size(x));

    % parab_coeff = [a; b; c; d; e; f];
    % z = a + b*x + c*y + d*x^2 + e*xy + f*y^2
    % z = [ons, x, y, x2, xy, y2] * parab_coeff
    
    
%     coeff_dbg = ([ons, x, y, x2, xy, y2] \ z)'
%     figure(2);
%     Origin_dgb = repmat(XYZ_1(t, :), size(XYZ,1), 1);
%     XYZ_dbg = (M * ((XYZ-Origin_dgb)'))';
%     trimesh(TRIV(tri_idx, :), XYZ_dbg(:,1), XYZ_dbg(:,2), XYZ_dbg(:,3));
%     grid on; xlabel('x'); ylabel('y'); zlabel('z');
    
    parab_coeff(t, :) = ([ons, x, y, x2, xy, y2] \ z)';
end % for t=1:num_tri
end % function [parab_coeff] = tri_parab_fit(TRIV, X, Y, Z)



