SHAPES = dir(fullfile(EVECS_DIR, FILES_TO_PROCESS));
SHAPES = {SHAPES.name};
%SHAPES = SHAPES(1:225); %SHAPES(226:end);
%SHAPES = SHAPES(1:2:end);
%SHAPES = SHAPES(186:230);


% Statistics
tic;
nerr  = 0;
nskip = 0;
nok   = 0;
fprintf(1, 'Computing diffusion distances...\n');


% Create directories
warning off;
for d = 1:size(DISTANCES,1),
    j = find(strcmpi(DISTANCES{d,3}, NORMS(:,2)));
    if isempty(j),
        continue;
    end
    normname = NORMS{j,2};
    kernel = DISTANCES{d,1};
    mkdir([DIFFDIST_DIR '.' kernel '.' normname]);
end
warning on;

for s = 1:length(SHAPES),
    
    shapename = SHAPES{s};
    fprintf(1, '  %-30s \t ', shapename);
    
    load(fullfile(EVECS_DIR, shapename), 'A');
    if min(size(A)) ~= 1, A = diag(A); end
    A = full(A);

    try
    
        for d = 1:size(DISTANCES,1),

            % Find corresponding norm
            j = find(strcmpi(DISTANCES{d,3}, NORMS(:,2)));
            if isempty(j),
                nerr = nerr+1;
                continue;
            end
            normfun  = NORMS{j,1};
            normname = NORMS{j,2};

            % Load kernel
            kernel = DISTANCES{d,1};

            if SKIP_EXISTING && exist(fullfile([DIFFDIST_DIR '.' kernel '.' normname], shapename), 'file'),
               nskip = nskip+1;
               continue;
            end          
            
 %           fullfile([DIFFDIST_DIR '.' kernel '.' normname], shapename)

            if exist('FAST_DISTANCE', 'var') && FAST_DISTANCE,
                % Fast distance computation - load eigenbasis
                % Skip if not L2 distance
                if ~strcmpi(lower(DISTANCES{d,3}), 'l2'), 
                    nskip = nskip+1;
                    continue; 
                end
                load(fullfile(EVECS_DIR, shapename), 'evecs', 'evals');                
                if size(evecs, 1) > MAX_VERTICES,
%                    fprintf(1, '[too many vertices]');
%                    return;
                    nskip = nskip+1;
                    continue; 
                end                
            else
                % Slow distance computation - load kernel
                load(fullfile([KERNEL_DIR '.' kernel], shapename), 'K', 'T');            
            end

            T0 = [];
            D  = {};
            for t = 1:length(DISTANCES{d,2}),

                % Find corresponding time scale
                if ~(exist('FAST_DISTANCE', 'var') && FAST_DISTANCE),
                    i = find(T==DISTANCES{d,2}(t));
                    if isempty(i), continue; end
                    time = T(i);
                else
                    time = DISTANCES{d,2}(t);
                end

                % Find kernel function
                k = find( strcmpi( KERNELS(:,2), DISTANCES{d,1} ) );
                if isempty(k), continue; end
                kernelfun = KERNELS{k,1};

                % Compute distance matrix
                if exist('FAST_DISTANCE', 'var') && FAST_DISTANCE,                
                    D{end+1}  = sqrt( mexdist(evecs, kernelfun(abs(evals),time).^2) );
                else
                    D{end+1}  = diffusiondist(K{i}, @(V)(normfun(V,A)));
                end

                T0(end+1) = time;

            end
            T = T0;

            save(fullfile([DIFFDIST_DIR '.' kernel '.' normname], shapename), 'D', 'T');
            nok = nok+1;

        end


        % Save result
        fprintf(1, 'OK\n');
        
    catch
        
        fprintf(1, 'Error\n');
        nerr = nerr+1;
        
    end
        
end

% Statistics
fprintf(1, '\nComputation complete\n');
fprintf(1, ' Elapsed time:   %s\n', format_time(toc));
fprintf(1, ' Total Shapes:   %d\n', length(SHAPES));
fprintf(1, ' Computed:       %d\n', nok);
fprintf(1, ' Skipped:        %d\n', nskip);
fprintf(1, ' Errors:         %d\n', nerr);

