% Error rates of for different distance functions
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.


% Fix vocabulary
v = find(strcmpi(cellfun(@chop_extension, VOCABS, 'UniformOutput', false), BOF_VOCAB));

% Positive & negative indices
M = MASK;
M(find(strcmpi(xform,'noise')),:) = 0;
M(:,find(strcmpi(xform,'noise'))) = 0;
[idxp,idxn] = posnegidx(M, true);

idxnull = find(strcmpi(xform, 'null'));

fprintf(1, 'Error rates for different distance functions\n');
fprintf(1, 'Vocab: %s\n', VOCABS{v});
for t = 1:length(BOFS{v}),
    for d = 1:length(BOF_DISTANCES),

        % compute idf weight
        idf = compute_idf(BOFS{v}{t},idxnull);
        
        if t==1,
            b = 'BoF';
            
            % Compute distance
            DIST = bofdist(BOFS{v}{t}, BOF_DISTANCES{d}, idf);
        else
            b = sprintf('SS-BoF (%.0f)', TIME_KERNEL(t-1));            
            
            % Compute distance
            %DIST = bofdist(BOFS{v}{t}, BOF_DISTANCES{d}, idf) + bofdist(BOFS{v}{1}, BOF_DISTANCES{d}, compute_idf(BOFS{v}{1},idxnull));
            DIST = bofdist(BOFS{v}{t}, BOF_DISTANCES{d}, idf);

        end
        fprintf(1, ' %-15s %-15s \t ',  b, BOF_DISTANCES{d});        


        % Compute distance
        %DIST = bofdist(BOFS{v}{t}, BOF_DISTANCES{d}, idf);

        % Compute rates
        [eer,fpr1,fpr01]= calculate_rates(DIST(idxp), DIST(idxn));
        fprintf(1, 'EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
           eer*100, fpr1*100, fpr01*100);

    end
end


