% mAP of SS-BoFs by class
%
% (C) Copyright Alex Bronstein, Michael Bronstein, Maks Ovsjanikov,
% Stanford University, 2009. All Rights Reserved.


% Fix vocabulary
v = find(strcmpi(cellfun(@chop_extension, VOCABS, 'UniformOutput', false), BOF_VOCAB));
%t = 2; % Remove hard-coding
t = 1;

% Output settings
fprintf(1, 'Error rates by class\n');
if t==1,
    b = 'BoF';
    dist = BOF_DISTANCE;
else
    b = sprintf('SS-BoF (%.0f)', TIME_KERNEL(t-1));
    dist = SSBOF_DISTANCE;
end
fprintf(1, 'LB: %s Vocab: %s  %s  Distance: %s\n', LB_PARAM, VOCABS{v}, b, dist);

% Compute distance
DIST = bofdist(BOFS{v}{t}, dist);




xforms = unique(xform);

idxQ = find(shapeid<=15);
ap = idxq*0;

for q=1:length(idxQ),

    idxq = idxQ(q);
    
            d = DIST(idxq,:);
            m = MASK(idxq,:);
            m(idxq) = 0;

            D = [D; d];
            M = [M; m];

            [d,idx] = sort(d, 'ascend');
            m = m(idx);

            n = cumsum(double(m~=0));
            r = cumsum(double(m>0));

            if r(end) <= 0, continue; end

            n(n==0 & r==0) = 1;
            pre = r./n; 
            rec = r./r(end);

            ap(q) = sum(double(m>0).*pre)/r(end);
end

for x=1:length(xforms),
    maxstrength = Inf;
%    if strcmpi(xforms{x}, 'noise'), maxstrength = 3; end
    idx = find(strcmpi(xform(idxQ), xforms{x}) & strength(idxQ) <= maxstrength);
    map = mean(ap(idx));
    fprintf(1, '   %-20s \t mAP = %5.2f%%\n', ...
        xforms{x}, map*100);    
end




return;

xforms = unique(xform);
idx1  = find(~strcmpi(xforms, 'null'));
xforms = xforms(idx1);
xforms{end+1} = 'all';

ERRORS = zeros(length(xforms),4);
for x=1:length(xforms),
   
    ap = [];
    D = [];
    M = [];
    for s = 1:15,
        
        idxq = find(shapeid == s & strength == 0);
        if isempty(idxq), continue; end
        
        idxd = setdiff(1:length(shapeid), idxq);
        %idxr = find(shapeid == s & strcmpi(xform, xforms{x}) & strength > 0);
        if strcmpi(xforms{x},'all'),
            idxr = find(shapeid == s & strength == 0);
        else
            idxr = find(shapeid == s & ~strcmpi(xform, xforms{x}));
        end
        idxr1 = find(strcmpi(xform, 'noise') & strength > 3);
        
        d = DIST(idxq,:);
        m = MASK(idxq,:);
        m(idxr) = 0;
        
        m(idxr1) = 0;
        
        D = [D; d];
        M = [M; m];
        
        [d,idx] = sort(d, 'ascend');
        m = m(idx);

        n = cumsum(double(m~=0));
        r = cumsum(double(m>0));
        
        if r(end) <= 0, continue; end
        
        n(n==0 & r==0) = 1;
        pre = r./n; 
        rec = r./r(end);

        ap(end+1) = sum(double(m>0).*pre)/r(end);
        
    end
    map = mean(ap);
    
    idxp = find(M>0);
    idxn = find(M<0);
    [eer,fpr1,fpr01] = calculate_rates(D(idxp), D(idxn));
    
    fprintf(1, '   %-20s \t mAP = %5.2f%% \t EER = %5.2f%% \t FAR@1%% = %5.2f%% \t FAR@0.1%% = %5.2f%%\n', ...
        xforms{x}, map*100, eer*100, fpr1*100, fpr01*100);    
end

