function SHAPES = xform_holes(shape, minholes, maxholes, maxstrength, flag)

% FPS
idx = fps(shape, maxholes);

% Maximum hole extent
D0 = repmat(Inf, [length(shape.X) 1]);
D0(idx) = 0;
D = fastmarch(shape.TRIV, shape.X, shape.Y, shape.Z, D0, struct('mode', 'single'));
D(isinf(D)) = NaN;
Dmax = max(D(:));
D(isnan(D)) = Dmax;

SHAPES = {};
for strength = 1:maxstrength,
    
    % Hole settings
    if maxstrength == 1
        lambda = 0;
    else
        lambda = (strength-1)/(maxstrength-1);
    end
    n  = round(minholes*(1-lambda)+maxholes*lambda);
    th = 0.1*(1-lambda) + 0.2*lambda;
    Dth = Dmax*th;
    i  = idx(1:n);

    % Create distance map
    D0 = repmat(Inf, [length(shape.X) 1]);
    D0(i) = 0;
    D = fastmarch(shape.TRIV, shape.X, shape.Y, shape.Z, D0, struct('mode', 'single'));
    D(isinf(D)) = Dmax;
    
    % Drill holes
    shape1 = geodesic_contour(shape, D, Dth);
    shape1 = reindex_mesh(shape1, [], find(shape1.tri_labels == 0));
    
    % find correspondence
    if flag
        [shape1.t,shape1.u] = find_corr(shape,shape1);
    end
    
    SHAPES{strength} = shape1;
end
