function SHAPES = xform_topology(shape, th1,th2,th3min,th3max,minpoints,maxpoints,minconn,maxconn, maxstrength, flag)

shape0 = shape;
shape1 = remesh(shape, struct('faces', 2000, 'verbose',0));
D = fastmarch(shape1.TRIV, shape1.X, shape1.Y, shape1.Z);
D_ = sqrt(abs(squared_dist([shape1.X shape1.Y shape1.Z])));

warning off
R = D_./D;
warning on

% maxstrength = 5;
% maxpoints = 5;
% th1 = 0.2;  % min geodesic distance
% th2 = 0.1;  % radius to remove around each point
% th3 = 0.025; % points around to connect

for strength = 1:maxstrength,
    
    shape = shape0;
    
    if maxstrength == 1
        lambda = 0;
    else
        lambda = (strength-1)/(maxstrength-1);
    end

    nconn  = round( maxconn*lambda+minconn*(1-lambda) );
    npoints  = round( maxpoints*lambda+minpoints*(1-lambda) );
    th3  = th3max*lambda+th3min*(1-lambda);

    MASK = ones(size(R));
    MASK(D<th1*max(D(:))) = NaN;
    MASK_ = zeros(size(R));

    TT = [];
    for k = 1:nconn;
        [mm,ii] = min(R(:).*MASK(:).*(1+MASK_(:)));
        [i,j] = ind2sub(size(R),ii);

        MASK_(find(D(i,:)<th2*max(D(:))),:) = NaN;
        MASK_(find(D(j,:)<th2*max(D(:))),:) = NaN;
        MASK_ = MASK_ + MASK_';

        % translate point indices to hi-res mesh
        [mm,ii] = min(sum(([shape.X shape.Y shape.Z]-repmat([shape1.X(i) shape1.Y(i) shape1.Z(i)],[length(shape.X) 1])).^2,2));
        [mm,jj] = min(sum(([shape.X shape.Y shape.Z]-repmat([shape1.X(j) shape1.Y(j) shape1.Z(j)],[length(shape.X) 1])).^2,2));

        D0 = repmat(Inf, [length(shape.X) 1]); D0(ii) = 0;
        d1 = fastmarch(shape.TRIV, shape.X, shape.Y, shape.Z, D0, struct('mode', 'single'));

        D0 = repmat(Inf, [length(shape.X) 1]); D0(jj) = 0;
        d2 = fastmarch(shape.TRIV, shape.X, shape.Y, shape.Z, D0, struct('mode', 'single'));


        i_ = find(d1<th3*max(D(:)));
        j_ = find(d2<th3*max(D(:)));

        TT = [TT; weld(shape,i_(1:min([npoints length(i_) length(j_)])),j_(1:min([npoints length(i_) length(j_)])))];
    end

    shape.TRIV = [shape.TRIV; TT];

    % find correspondence - trivial
    if flag
        [shape.t,shape.u] = find_corr(shape);
    end
    
    
    SHAPES{strength} = shape;
end


return
